pro gbo_obs_coord, kind, gbotime=gbotime, obstime, gboimage, window=window, $
  sindex=sindex, sdata=sdata, suncenter=suncenter, side=side, grid=grid, $
  color1=color1, color2=color

;+
;NAME:
;	gbo_obs_coord
;PURPOSE:
;	To print out the OR table coordinate for a region clicked on 
;	a rescaled GBO image
;PROCEDURE
;	1.  If sindex and sdata not passed in then get and display last SFD.
;	2.  Get desired GBO image.
;	3.  If needed rotate or flip GBO image.
;	4.  Determine center and radius of GBO image with CLICKLIMB
;	5.  TVSCL rescaled GBO image onto last SFD.
;	6.  Program calls SXT_OBS_COORD to derive table coordinates.
;CALLING SEQUENCE:
;	gbo_obs_coord, kind, obstime [, data=data, side=side, /grid]
;	gbo_obs_coord, 'g*h',gbotime='29-nov-94', obstime, side=128  ; 2x2 OR
;	gbo_obs_coord, 'gbh', obstime, sindex=index, sdata=data, $
;		window=4, side=[64,128]  ; 1x2 OR (NS x EW)
;       gbo_obs_coord, 'gki', '3-MAY-95  16:40', gboimage
;INPUT:
;	kind	- type of GBO image desired, e.g., 'gbh','gkm','g*h, etc.
;	          G_x   Ground-Based Observation (GBO) Images (Sites)          
;                 Gx_   Ground-Based Observation (GBO) Images (Data Type)
;                 GBC   GBO Big Bear Large Scale H-alpha PFI FITS Image
;                 GBH   GBO Big Bear H-alpha FITS Image    
;                 GBK   GBO Big Bear Calcium FITS Image
;                 GBW   GBO Big Bear White Light FITS Image
;                 GCH   GBO Boulder Colorado H-alpha FITS Image  
;                 GGH   GBO GSFC H-alpha FITS Image  
;                 GHH   GBO Holloman H-alpha FITS image  
;                 GLH   GBO Leamonth Australia H-Alpha FITS Image
;                 GNH   GBO NAOJ Japan H-alpha FITS Image  
;                 GJH   GBO CRL Hiraiso H-alpha FITS Image  
;                 GKI   GBO Kitt Peak He 10830 FITS Image   
;                 GKM   GBO Kitt Peak Magnetogram FITS Image   
;	obstime	- Time in any format at which coordinates of chosen point
;		  are desired (index, etc)
;OPTIONAL OUTPUT:
;	gboimage - scaled, positioned gbo image without pfi boxes.
;KEYWORDS (INPUT):
;	sindex	- index for SXT image (not required if lastsfd is OK).
;	sdata	- SXT image to be displayed (not required if you don't
;		  want to see it or if sindex is set.)
;	gbotime	- Desired time of image of desired GBO image in any format (index, etc)
;		  Default: index of lastsfd image or sindex
;	window	- The window in days +/- the input time which is accecptable
;                 to search for an GBO image.  Default = 8.
;		  In this case program searches for the GBO image
;		  nearest in time to the time given by gbotime.
;	suncenter - Center of sun in full res (IDL) pixels.  
;		    Defaults to suncenter=sxt_cen(gbotime)
;	side	- Size of square in FR pixels to draw on image (def = 64). If 
;		  side =[N,M], will draw a rectangle that is N (NS) by M (EW)
;	grid	- If set, overlay heliographic coordinate grid
;	color1	- Color of ROI box (def = 192).
;	color2	- Color of rotated ROI box (def = 255).
;HISTORY:
;	30-Nov-94 - LWA - Written.
;	2-May-95  - LWA - renamed variables and arranged to use 
;			sindex and sdata in place of lastsfd if
;			desired.
;	4-May-95  - LWA - Added ability to return GBO image, replaced
;			get_xyrad with clicklimb, and removed HR limitation.
;	15-May-95 - LWA - Corrected header.
;-

help,gbotime,obstime

wdef,20,/uright

if (keyword_set(sdata) and keyword_set(sindex)) then tvscl,sdata
if NOT keyword_set(sindex) then lastsfd,sindex,sdata
res=gt_res(sindex)

if NOT keyword_set(window) then window=8 else window=window
if NOT keyword_set(gbotime) then gbotime=sindex else gbotime=gbotime
gbimg=get1gbo(kind,gbotime,dattim=dattim,window=window)
siz=size(gbimg)
wdef,21,siz(1),siz(2),/uleft
tv,gbimg

print,'   Does image need to be rotated to put north up and east left?'
print,'   If so, enter a direction according to the following table:'
print
print,'  Direction  Transpose?   Rot. CCW    X1  Y1
print,'      ----------------------------------------
print,'      0          No          None     X0  Y0
print,'      1          No          90      -Y0  X0
print,'      2          No          180     -X0 -Y0
print,'      3          No          270      Y0 -X0
print,'      4          Yes         None     Y0  X0
print,'      5          Yes         90      -X0  Y0
print,'      6          Yes         180     -Y0 -X0
print,'      7          Yes         270      X0 -Y0
print    
print,'  The notation -Y0 indicates a reversal of the Y axis.  [Default = 0]

input,'  Direction:  ',flipper,0,0,7
temp=rotate(gbimg,flipper)
tv,temp

yesnox,'  Is this OK?  If not, enter NO and get a second chance.',ok_nok,'yes'

if (ok_nok eq 0) then begin
   input,'  Direction:  ',flipper,0,0,7
   temp=rotate(gbimg,flipper)
   tv,temp
endif

gbimg=temp & temp=0

xy=sxt_cen(sindex)
hrad=get_rb0p(dattim,/radius)

print
print,'   Click several places around the limb (minimum 3 places)'
print,'   with LH mouse button.  RH button when finished.'
print

xyr=clicklimb(gbimg)

yesnox,'  Is this OK?  If not, enter NO and get a second chance.',ok_nok,'yes'

if (ok_nok eq 0) then begin
   xyr=clicklimb(gbimg)
endif

x=xyr(0) & y=xyr(1) & r=xyr(2)
print,'x = ',x,'  ','y = ',y,'  ','r = ',r
hscale=(r/(hrad*res/gt_pix_size(sindex)))  ; Always scale QR to 512x512.

wset,20
tvscl,congrid(gbimg,siz(1)/hscale,siz(2)/hscale,/interp), $
    (xy(0)/2)-(x/hscale),(xy(1)/2)-(y/hscale)
if keyword_set(grid) then sxt_grid,sindex

xyouts,/dev,50,55,'UT of GBO image:  '+fmt_tim(dattim),siz=1.5
xyouts,/dev,50,40,'UT of Obs Time:    '+ obstime,size=1.5

yesnox,'  Do you wish to adjust the color table?',xct,'no'
if (xct eq 1) then begin
   xloadct
endif
gboimage=tvrd()

if NOT keyword_set(side) then side=64 else side=side
if NOT keyword_set(color1) then color1=1 else color1=color1
if NOT keyword_set(color2) then color2=255 else color2=color2
if NOT keyword_set(suncenter) then suncenter=[xy(0),xy(1)] $
     else suncenter=suncenter

sxt_obs_coord,fmt_tim(dattim),obstime,suncenter=suncenter,res=1, $
   color1=color1,color2=color2,side=side

end
