;+
; NAME:
;       GOES_REDUCER
; PURPOSE:
;       Reduce GOES data to T, em, luminosity, etc. Note that the default
;         call gets 3-second data from the GOES archive. These data, while
;         good, often have "range change" glitches arising in the GOES
;         analog electronics. Right now these must be dealt with manually
;         if one needs a clean time series.
; CATEGORY:
; CALLING SEQUENCE:
;       goes_reducer, t1, t2, goes_time, goes_out, total_energy 
;       goes_reducer, t1, t2, goes_time, goes_out, total_energy, /three
;       goes_reducer, t1, t2, goes_time, goes_out, total_energy, /noback,/plot
;	goes_reducer, t1, t2, goes_time, goes_out, /notemps
;       goes_reducer, t1, t2, times, outs, /three, /fix_time, /noback, /quiet
; INPUTS:
;       t1 = start time (formatted, eg '15-nov-91 22:00')
;       t2 = end time (formatted)
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;       NOBACK defeats the background estimate 
;         (ie, takes the total signal)
;       FIRST_BACK sets the background level to the first point (t1)
;       PLOT makes standard plots (done indepently with GR_SUMMARY_PLOT)
;       FIX_TIME bypasses the PLOT_LCUR calls, i.e. non-interactive
;       QUIET defeats message-printing (the "Blame Roger Thomas" one)
;	NOTEMPS just returns data (can be used with /noback just to get
;	  the GOES fluxes, as an alternative to plot_goes)
;       ONE reverts to one-minute integrations
;       FIVE gives five-minute integrations
;       THREE prints a note about the default 3-second data.
; OUTPUTS:
;       goes_time is an index-type file of times, for UTPLOT entry
;       goes_out is a structure containing the fit parameters:
;          temperatures (MK)
;          emission measures (units of 10^55 cm^-3)
;          total radiated powers (ergs/s)
;          radio ff flux (sfu @ 15 GHz. This assumes He/H = 6.3% abundance)
;          GOES fractional fluxes (high-energy channel, low-energy channel)
;            (these are the energy flux in band normalized to the total)
;          GOES input fluxes (w/m^2; these are background-corrected)
;          GOES backgrounds as input interactively
;          terrmin is lower limit for temperature
;          terrmax is upper limit for temperature
;          emerrmin is lower limit for emission measure 
;          emerrmax is upper limit for emission measure
;       total_energy is the time integral of X-ray radiated energy
; WARNING:
;       The present GOES data sometimes have mismatched points in the
;         different channels. This causes a 'dischronicity alert' and
;         a halt.
;       Beware of results if the signal-to-background ratio is small.
;       The total power fit is valid only for 5-30 MK and doesn't
;	  include long-wavelength emissions of probable significance,
;	  such as Ly-alpha or He 304.
;       With the 3-sec GOES data, the "range change" glitches are
;         quite obvious. These must currently be dealt with manually.
;       None of this has been tried on GOES-8 data
; MODIFICATION HISTORY:
;       HSH written 21-Feb-93
;       HSH added radio fluxes (algorithm from Stephen White, U. Md.) 13-Mar-93
;       HSH added fractional energy flux outputs 30-Mar-93
;       HSH added /three keyword 18-Apr-93
;       HSH added /noback and fixed other things 24-Apr-93
;       PLB added error bars to the calculation 9/13/93.  see ApJ reference.
;           this uses the three extremes of including and subtracting 
;           background fluxes; it does not fit the rise of the flare as
;           described in the reference (Bornmann, P. 1990, Ap. J. 356, p. 733).
;	HSH added /plot keyword 9-Oct-93
;	HSH added /notemps keyword 22-Nov-93
;       HSH added /fix_time, /quiet, /first_back 11-May-95
;       HSH converted to RD_GXD, 3-sec default, 2-Dec-95 (/one, /five)
;       HSH fixed the radio flux algorithm (thanks for bug report from
;           K. Shibasaki, NRO), 26-Apr-96
;-
pro goes_reducer, t1, t2, goes_time, goes_out, total_energy, one = one, $ 
  five = five,  noback = noback, plot = plot, notemps = notemps,$
  fix_time = fix_time,  quiet = quiet, first_back = first_back, three = three

if keyword_set(three) then begin
  print,''
  print,'GOES_REDUCER now assumes 3-sec data unless /one or /five are set'
  print,'GOES_REDUCER now assumes 3-sec data unless /one or /five are set'
  print,'GOES_REDUCER now assumes 3-sec data unless /one or /five are set'
  print,'GOES_REDUCER now assumes 3-sec data unless /one or /five are set'
  print,'GOES_REDUCER now assumes 3-sec data unless /one or /five are set'
  print,''
endif
rd_gxd, t1, t2, goes
hidata = goes.hi
lodata = goes.lo
index = goes
if keyword_set(one) then begin
  rd_gxd, t1, t2, goes, /one
  hidata = goes.hi
  lodata = goes.lo
  index = goes
endif
if keyword_set(five) then begin
  rd_gxd, t1, t2, goes, /five
  hidata = goes.hi
  lodata = goes.lo
  index = goes
endif


;	interactive data selection

if n_elements(fix_time) eq 0 then begin
  print,' '
  print,'SELECT DATA INTERVAL NOW'
  print,' '
  ss = plot_lcur(index,lodata,tit='GOES low channel: Select for reduction')
  print,' '
  if keyword_set(noback) eq 0 then begin
    print,'SELECT BACKGROUND INTERVAL NOW'
    print,' '
    ssb = plot_lcur(index,lodata,tit='GOES low channel: Select for background')
    n_back = n_elements(ssb)
  endif
endif else begin
  ss = indgen(n_elements(lodata))
endelse
hi = hidata
lo = lodata
;
; simpler treatments of background fluxes 
;
if keyword_set(first_back) then begin
  ssb = indgen(5)
  ss = indgen(n_elements(lo)-5) + 5
  n_back=5
endif
if keyword_set(noback) eq 0 then begin
  loback = total(lo(ssb))/n_back
  hiback = total(hi(ssb))/n_back
endif
if (keyword_set(noback) ne 0) then begin
  loback = 0
  hiback = 0
endif

hi = hi(ss)
lo = lo(ss)
index = index(ss)
n_data = n_elements(index)
if n_elements(quiet) eq 0 then begin
  print,' '
  print,'Background rates are: ', hiback, loback
  print,' '
endif
loback = replicate(loback,n_data)
hiback = replicate(hiback,n_data)
goes_out = {goes_params, temp:0.0, em:0.0, power:0.0, radio:0.0, $
  fluxes:[0.,0.],fracs:[0.,0.],backs:[0.,0.], $
  terrmin:0.0, terrmax:0.0, emerrmin:0.0, emerrmax:0.0}
terrmin  = fltarr(n_data)
terrmax  = fltarr(n_data)
emerrmin = fltarr(n_data)
emerrmax = fltarr(n_data)
goes_out  = replicate(goes_out,n_data)
goes_time = index
for i = 0, n_data-1 do begin
  ; put in two more calls to thomas_goes here, leaving in all the 
  ; flux, and subtracting the low energy background while keeping
  ; all of the high energy flux
  if (keyword_set(notemps) eq 0) then begin
    temps0 = thomas_goes (hi(i)          , lo(i)          ,quiet=quiet)
    temps1 = thomas_goes (hi(i)          , lo(i)-loback(i),quiet=quiet)
    temps  = thomas_goes (hi(i)-hiback(i), lo(i)-loback(i),quiet=quiet)
  endif else begin
    temps = [0,0] 
    temps0 = [0,0] 
    temps1 = [0,0]
  endelse
  

  ; Save the error limits
  trange  = [temps(0), temps0(0), temps1(0)]
  emrange = [temps(1), temps0(1), temps1(1)]
  goes_out(i).terrmin  = min (trange)
  goes_out(i).terrmax  = max (trange)
  goes_out(i).emerrmin = min (emrange)
  goes_out(i).emerrmax = max (emrange)

  goes_out(i).temp = temps(0)
  goes_out(i).em   = temps(1)
  if (keyword_set(notemps) eq 0) then begin
    power = thermal_power(temps(0))*temps(1)*1e11
    goes_out(i).power = power 
    goes_out(i).radio = 1.577e+06*temps(1)/sqrt(temps(0)) * $
;     (24.2+alog(temps(0))-alog(15.0e+09))
      (24.5+alog(temps(0)*1.e+06)-alog(15.e+09))
    flux = power*1e-4 / 2.81e27
    goes_out(i).fracs(0)  = (hi(i)-hiback(i))/flux
    goes_out(i).fracs(1)  = (lo(i)-loback(i))/flux
  endif else begin
    goes_out(i).power = 0
    goes_out(i).radio = 0
    goes_out(i).fracs(0) = 0
    goes_out(i).fracs(1) = 0
  endelse
  goes_out(i).fluxes(0) = hi(i)-hiback(i)
  goes_out(i).fluxes(1) = lo(i)-loback(i)
  goes_out(i).backs(0)  = hiback(i)
  goes_out(i).backs(1)  = loback(i)
endfor
numb = n_elements(index)
dtimes = index(1:numb-1).time-index(0:numb-2).time
dtimes = [dtimes,dtimes(numb-2)]
total_energy = total(goes_out.power * dtimes / 1000.) 

; Make plots of temperature and emission measure results
; plot_range, goes_time, goes_out
if (keyword_set(plot) ne 0) then gr_summary_plot, goes_time, goes_out 


;stop
end

