;+
;       NAME: limbpnts
;
;
;       PURPOSE: Find limb points for spectral images
;
;       METHOD:  Take estimated center and take rays of points.
;		 Call CUT_LEV to estimate background and subtract. Take 
;		 first non-zero values as the limb. 
;		 When there are spurious points, either inside or outside
;		 the limb they are left out. If there are many increasing NP is 
;		 recomemended. Adjusting them tended to through of the ellipse 
;		 fitter.
;		 When the limb is cropped, points on the image edge are not 
;		 returned.
;
;       CALLING SEQUENCE: limb=limbpnts(solar,cent,np,/quiet)
;
;
;       PARAMETERS:	solar	input image
;			cent	estimated image centroid
;			np	number of points to select
;			limb	limb points returned in [x,y] form. 
;				NOTE: this is not necessarily the same number as NP.
;
;	KEYWORDS:	quiet	when NOT set display image and points
;
;       HISTORY:	drafted by A.McAllister nov. 1993
;			corrected the limb search to come in from the outside, AMcA, mar. 1995
;			reworked to compare inward and outward sweeps, remove mismatches, AMcA, apr. 1996
;			changed to leave out cropped points, AMcA, apr. 1996.
;
;	CAUTION:  	This new version should be checked on various data.
;
;-
FUNCTION limbpnts,solar,cent,np,quiet=quiet

image=solar						;buffer input
imgsz=size(image)

cut=cut_lev(image,imgsz)				;remove background
image=image-cut

maxlen=sqrt(imgsz(1)*imgsz(1) + imgsz(2)*imgsz(2)) 	;image diagonal

dthet=361./np
theta=findgen(np) * dthet * !dtor			;angles

;ex=cos(theta)*maxlen + cent(0)    			;end points
;ey=sin(theta)*maxlen + cent(1)

;for each line

line = lindgen(maxlen)

lx=((cos(theta)#line + cent(0)) > 0) < imgsz(1)		;get points on line
ly=((sin(theta)#line + cent(1)) > 0) < imgsz(2)

lx(where(lx le 0. or lx ge imgsz(1))) = 0		;tag out of bound points
ly(where(ly le 0. or ly ge imgsz(2))) = 0
lx(where(ly eq 0)) = 0					
ly(where(lx eq 0)) = 0

lnsize=size(lx) & linnmb=lnsize(1) & linlth=lnsize(2)
lxy=fltarr(2,linnmb*linlth)
lxy(0,*)=lx & lxy(1,*)=ly				;reorganize for conversion
linss=coord_v2l(lxy,imgsz)
linss=reform(linss,linnmb,linlth,/overwrite)		;linear coords by line

limb=intarr(2,linnmb)

if not keyword_set(quiet) then tvscl,image

for ii = 0,linnmb-1 do begin

 rlin=linss(ii,*)
 tlin=rlin(where(linss(ii,*) gt 0))			;crop outside-the-image zeros
 
 disk=where(image(tlin) ge (max(solar)-cut)*.025)       ;pad the cut value slightly 
							;for processing noise at the limb

 other=where(image(tlin) le (max(solar)-cut)*.025)	;take the negative image

 if other(0) eq (-1) then begin

    lind= 0						;no limb point, flag cropped disk 

 endif else begin

    last_disk = n_elements(disk)-1			;the outermost point above the cutoff
    first_other = other(0)				;the innermost point below the cutoff

    incriment = 1					;counter

    if (first_other - last_disk) lt 0 then lind=0 else lind=last_disk   ;seems better to simply 
									;remove the points completely
 
    ;while (first_other - last_disk) lt 0 do begin 	;if you want to try to keep them....

    	;last_disk = n_elements(disk)-(incriment + 1)	;move to next innerward point
    	;first_other = other(incriment)			;move to next outerward point
    	;incriment = incriment + 1			;augment the counter

    ;endwhile					
    ;lind = last_disk					;set the value
   
 endelse
  
 if lind eq 0 then limb(*,ii) = [-1,-1] else limb(*,ii)=coord_l2v(linss(ii,lind),imgsz)	;flag cropped points
  
endfor

edge = where(limb(0,*) ne -1)

limb = limb(*,[edge])

npnts = n_elements(limb(0,*))

if not keyword_set(quiet) then for ii = 0,npnts-1 do plots,limb(0,ii),limb(1,ii),psym=1,/device,color=255

return, limb

end
