pro pr_nar, st_date, en_date, qstop=qstop, outfile=outfile, hc=hc, outstring=outstring, $
		noprint=noprint, nearest=nearest
;
;+
;NAME:
;	pr_nar
;PURPOSE:
;	To list the NOAA Active Region list
;CALLING SEQUENCE:
;	pr_nar, '1-jan
;	pr_nar, '30-oct-91', '2-nov-91', /hc
;INPUT:
;	st_date	- The JST date to search for.  If no date is entered,
;		  the current date is used.  The year is not
;		  required, but if entered should be of the
;		  form 1-Oct-91 style.  The date should be
;		  entered in string style with date first.
;OPTIONAL INPUT:
;	en_date	- The ending date to list data for.  If no date is passed,
;		  it only searches for "st_date"
;OPTIONAL KEYWORD INPUT:
;	outfile	- If a filename is passed, the results will also
;		  be written to that file
;	hc	- If set, write the results to a temporary file, print
;		  it and delete it.
;	noprint- If set, do not print the results to the screen
;	nearest- Passed on to RD_NAR
;OPTIONAL KEYWORD OUTPUT:
;	outstring- The results of the NAR listing in a string array
;OUTPUT:
;	The times and classes of the GOES events
;HISTORY:
;	Written 28-Jan-93 by M.Morrison
;	 8-Apr-93 (MDM) - Added OUTSTRING and NOPRINT options
;			- Added NEAREST option
;-
;
qhc = 0
qout = 0
if (keyword_set(outfile)) then begin
    openw, lunout, outfile, /get_lun
    qout = 1
end
if (keyword_set(hc)) then begin
    openw, lunhc, 'pr_nar.temporary', /get_lun
    qhc = 1
end
qprint = not keyword_set(noprint)
;
if (n_elements(st_date) eq 0) then begin
    st_date = strmid(!stime, 0, 6)
end else begin
    ; If the date was provided, make sure to add a yr if none provided:
    day_arr = timstr2ex(st_date)
    if day_arr(6) eq 0 then day_arr(6) = fix(strmid(!stime,9,2))
    dummy = fmt_tim(day_arr, day_str, time_str)
    st_date = day_str
end
;
day_arr1 = timstr2ex(st_date)
;
if (n_elements(en_date) eq 0) then begin
    day_arr2 = anytim2ex(anytim2ints(st_date, off=23.5*60*60))		;just end 23.5 hours later
end else begin
    day_arr2 = timstr2ex(en_date)
end
;
rd_nar, fmt_tim(day_arr1), fmt_tim(day_arr2), nar_data, nearest=nearest, status=status
if (status gt 0) then begin
    message, 'Cannot find any NAR data', /info
    return
end
;
tit = strarr(4)
tit(0) = 'PR_NAR.PRO Run on: '+ !stime
tit(1) = '  '
tit(2) = '   Date     Number Location Long   Area   Z    LL    NN  MagType
tit(3) = '  '
         ;  8-MAY-92    7151  S09W39  207      0   AXX    0     1  ALPHA
outstring = tit
;
for i=0,n_elements(tit)-1 do begin
    if (qprint) then print, tit(i)
    if (qhc) then printf, lunhc, tit(i)
    if (qout) then printf, lunout, tit(i)
end
;
ew_arr = ['E', 'W']
ns_arr = ['S', 'N']
;
qfirst = 1
for i=0,n_elements(nar_data)-1 do begin
    nar = nar_data(i)
    z = string(nar.st$macintosh)
    mag_type = string(nar.st$mag_type)

    tim1 = gt_day(nar, /str)
    location = '      '
    if (nar.location(0) ne -999) then location = string( ns_arr( nar.location(1) ge 0 ), abs(nar.location(1)), $
						ew_arr( nar.location(0) ge 0 ), abs(nar.location(0)), format='(2(a,i2.2))')

    str = string(tim1, nar.noaa, location, nar.longitude, nar.area, $
			z, nar.long_ext, nar.num_spots, mag_type, $
			format='(1x, a, 4x, i4, 2x, a, 2x, i3, 3x, i4, 3x, a, 3x, i2, 3x, i3, 2x, a)' )

    if (qprint) then print, str 
    if (qhc) then printf, lunhc, str
    if (qout) then printf, lunout, str
    outstring = [outstring, str]
end
;
if (qhc) then begin
    free_lun, lunhc
    dprint, 'pr_nar.temporary', /delete
end
if (qout) then free_lun, lunout
;
if (keyword_set(qstop)) then stop
end

