function rd_goes_fits, infil0, sttim, entim, rm_bad=rm_bad, force_read=force_read, qstop=qstop, $
		indir=indir, spacecraft=spacecraft
;+
;NAME:
;	rd_goes_fits
;PURPOSE:
;	Read the GOES FITS data file
;OPTIONAL INPUT:
;	infil	- The data file name.  If not passed, then the name can be
;		  derived from "sttim", "indir", and "spacecraft"
;	sttime	- The starting time to extract
;	entim	- The ending time to extract
;OPTIONAL KEYWORD INPUT:
;	rm_bad	- If set, then remove the bad datapoints (negative counts)
;	indir	- The input directory for the data file
;	spacecraft - Used when generating the input file name.  It should be
;		  the integer 6 or 7.
;OUTPUT:
;	returns the data structure
;OPTIONAL KEYWORD OUTPUT:
;	outrec	- The Yohkoh data structure with the information in the
;		  data file.
;HISTORY:
;	Written 12-Oct-93 by M.Morrison (using RD_GOES_CDROM as a start)
;	13-Oct-93 (MDM) - Allowed capability of generating the filename from
;			  the input times.
;	26-Jan-95 (MDM) - Changed input file creation.  Made it upper case
;			  and appended a ";"
;			- Also changed to use FSTAT instead of FILE_INFO
;-
;
common rd_goes_fits, last_infil, tim_map, ref_tim, nevery
;
noswap_os=['vms','ultrix','OSF']                ; list of noswap (extensible)
chk=where(!version.os eq noswap_os,nscount)     ; current OS in noswap list?
qswap = nscount ne 0                            ; assign boolean (opposite of RDWRT)
;
if (n_elements(infil0) eq 0) then begin
    tarr = anytim2ex(sttim)
    filnam = string('x', spacecraft, '3', tarr(6), tarr(5), '.fit', format='(a,i2.2,a,2i2.2,a)')		;x069208.fits
    filnam = strupcase(filnam) + ';1'	;MDM added 25-Jan-95
    infil = concat_dir(indir, filnam)
    infil = infil(0)	;make scalar
end else begin
    infil = infil0
end
;
if (not file_exist(infil)) then begin
    print, 'RD_GOES_FITS: Input file: ', infil, ' does not exist.  Returning'
    return, 0b
end
;
;;nfil = file_info(infil, info)
openr, lun, infil, /get_lun
info = fstat(lun)
free_lun, lun

ndrec = (info.size - 8*2880)/12			;there are 8 header blocks
ndrec = ndrec - 240				;there is a 2880 byte header at the end of the file (!??!) 240 = 2880/12
;
if (n_elements(last_infil) eq 0) then last_infil = 'xxx'
if (keyword_set(force_read)) then last_infil = 'xxx'
if (infil ne last_infil) then begin
    print, 'RD_GOES_FITS: Mapping the times in the files'
    nevery = 5000L
    n = ndrec/nevery
    tim_map = anytim2ints(!stime, off=intarr(n))
    for i=0,n-1 do begin
	data = readfits(infil, header, start=i*nevery, numr=1, /exten, /silent)
	;
	if (i eq 0) then begin
	    tarr = intarr(7)
	    ss = where(strmid(header,0,8) eq 'DATE-OBS')
	    tarr(4) = fix(strmid(header(ss(0)), 11, 2))
	    tarr(5) = fix(strmid(header(ss(0)), 14, 2))
	    tarr(6) = fix(strmid(header(ss(0)), 17, 2))
	    ref_tim = anytim2ints(tarr)
	end
	secs = data(0:3)		& secs = long(temporary(secs), 0)		& if (qswap) then dec2sun, secs
	tim_map(i) = anytim2ints(ref_tim, off=float(secs))
    end
    last_infil = infil
end
;
strec = 0
if (n_elements(sttim) ne 0) then begin
    ss = sel_timrange(tim_map, sttim, entim)
    strec = ss(0)*nevery
    if (max(ss) eq n_elements(tim_map)-1) then enrec = ndrec-1 else enrec = ((max(ss)+1)*nevery) < (ndrec-1)
    ndrec = enrec-strec+1
    print, 'RD_GOES_FITS: Starting,ending record: ', strec,enrec, ' for ', ndrec, ' records'
end
;
data = readfits(infil, header, numr=ndrec, start=strec, /exten)
;
nn = n_elements(data(0,*))	;not always equal to ndrec !?!?!
secs = data(0:3,*)		& secs = long(temporary(secs), 0, nn)		& if (qswap) then dec2sun, secs	
times = anytim2ints(ref_tim, off=float(secs))
;
cnts_short = data(8:9, *)	& cnts_short = fix(temporary(cnts_short), 0, nn)	& if (qswap) then dec2sun, cnts_short
cnts_long = data(10:11, *)	& cnts_long = fix(temporary(cnts_long), 0, nn)	& if (qswap) then dec2sun, cnts_long
;
gbo_struct, gxd_data=data_ref
outrec = replicate(data_ref, nn)
;
outrec.time	= times.time
outrec.day	= times.day
outrec.hi	= 10.^(cnts_short/1000.)	
outrec.lo	= 10.^(cnts_long/1000.)
;
if (n_elements(sttim) ne 0) then begin
    ss = sel_timrange(outrec, sttim, entim, /bet)
    if (ss(0) eq -1) then begin
	print, 'RD_GOES_FITS: No data found for the time range specified'
	outrec = 0b
    end else begin
	outrec = outrec(ss)
    end
end
;
if (keyword_set(rm_bad) and (get_nbytes(outrec) gt 10) ) then begin
    ss = where( (outrec.lo lt 32000) and (outrec.hi lt 32000) )
    if (ss(0) ne -1) then outrec = outrec(ss) $
	else outrec = 0b
end
;
if (keyword_set(qstop)) then stop
return, outrec
end
