pro rd_gxt, input1, input2, gxt_data, indir=indir, status=status, $
	nearest=nearest, vnum=vnum, full_weeks=full_weeks, qdebug=qdebug, $
	convert=convert, clean=clean
;
;+
;NAME:
;	rd_gxt
;PURPOSE:
;	Read the GXT files (GOES 1 minute average light curve data files
;	which were derived from the tektronics plot files)
;CALLING SEQUENCE:
;	rd_gxt, roadmap(0),  roadmap(n), gxt_data
;	rd_gxt, '1-dec-91', '30-dec-91', gxt_data
;	rd_gxt,      weeks,       years, gxt_data
;INPUT:
;		  INPUT CAN BE OF TWO FORMS
;
;		  (A) Input starting and ending times
;	input1	- Starting time in either (i) standard string format,
;		  or (ii) a structure with .time and .day fields
;		  (the 7 element time vector form is not allowed)
;	input2	- Ending time.  If ending time is omitted, the
;		  ending time is set to 24 hours after starting time.
;
;		  (B) Input can be a vector of week/year number
;	input1	- a vector of the week numbers to read
;	input2	- a vector of the year of the week to be read
;		  if the weeks vector is all within one year, the
;		  year parameter can be a scalar.
;OUTPUT:
;	data	- the data structure containing the data in the files
;		  See the structure definition for further information
;OPTIONAL KEYWORD INPUT:
;	indir	- Input directory of data files.  If not present, use
;		  $DIR_GEN_xxx logical directory
;	vnum	- The file version number to use.  If not present, a call
;		  to WEEKID is made and that latest version is used.
;	nearest	- If set, then the time span is adjusted a day at a time
;		  until it finds some data.  It decrements the starting
;		  date by a day and increments the ending date by a day
;		  up to 14 days (28 day total span)
;       full_weeks - If set, then do not extract the entries that just
;                 cover the times covered in the start/end time.  Return
;                 all data for weeks covered by the start/end time. This
;                 allows a user to have start and end time be the same
;                 and still get some data.
;	convert	- If set, then convert the arbitrary scale to real data
;		  values.  If you do not use /CLEAN, then you will need
;		  to filter out all values LE 1e-10
;	clean 	- If set, then remove all instances when the value is -1
;		  in any of the 4 channels.  
;OPTIONAL KEYWORD OUTPUT:
;	status	- The read status
;		  Some data is available if (status le 0)
;			 0 = no error
;			 1 = cannot find the file
;			 2 = cannot find data in the time period
;			-1 = found data, but had to go outside of the requested
;			     period (only true if /NEAREST is used).  
;HISTORY:
;	Written Jun-92 by M.Morrison
;	15-Apr-93 (MDM) - Replaced specific RD_GXT with call to RD_WEEK_FILE
;	17-Dec-93 (MDM) - Modified to return a real*4 structure with true
;			  data values rather than a scaled "TEK" value
;-
;
rd_week_file, input1, input2, 'GXT', gxt_data, $
	vnum=vnum, indir=indir, nearest=nearest, status=status, $
	full_weeks=full_weeks, qdebug=qdebug
;
if (keyword_set(convert) and (status le 0)) then begin
    if (keyword_set(clean)) then begin
	ss=where((gxt_data.g6hi ne -1) and (gxt_data.g6lo ne -1) and (gxt_data.g7lo ne -1) and (gxt_data.g7hi ne -1), n)
	if (n eq 0) then begin
	    print, 'RD_GXT: 
	    status = 3
	    gxt_data = 0b
	    return
	end
	gxt_data = gxt_data(ss)
    end
    ;	    
    n = n_elements(gxt_data)
    out = replicate({gxr2_data_rec}, n)
    out.time = gxt_data.time
    out.day  = gxt_data.day
    out.g6lo = 10. ^ ( float(gxt_data.g6lo-101) / (679.-101.) * 6 - 9)
    out.g6hi = 10. ^ ( float(gxt_data.g6hi-101) / (679.-101.) * 6 - 9)
    out.g7lo = 10. ^ ( float(gxt_data.g7lo-101) / (679.-101.) * 6 - 9)
    out.g7hi = 10. ^ ( float(gxt_data.g7hi-101) / (679.-101.) * 6 - 9)
    out.digitize = 2
    ;
    gxt_data = temporary(out)
end
;
end
