pro wfits,data,filnam,index=fnum,date_obs=date,time_obs=time, $
                      history=his,comment=com

;+
; NAME:
;        WFITS
; PURPOSE:
;        Writes an array into a standard FITS file on the disk.
;        A simple FITS header is created automatically.
; CATEGORY:
;        Input/Output
; CALLING SEQUENCE:
;        wfits, array, filename
; INPUTS:
;        array = data array.
;        filename = string containing the file name.
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;        index = nonnegative integer. If this parameter is present, a period
;              and the index number are appended to the filename (e.g., '.34').
;              This option makes handling of data in the MCCD file naming
;              convention easier.
;        date_obs = string. A FITS keyword parameter "DATE_OBS= 'string'" is
;                   added to the header.
;        time_obs = string. A FITS keyword parameter "TIME_OBS= 'string'" is
;                   added to the header.
;        history = string. A FITS keyword parameter "HISTORY string" is added
;                   to the header.
;        comment = string. A FITS keyword parameter "COMMENT string" is added
;                   to the header.
; OUTPUTS:
;        No explicit outputs.
; COMMON BLOCKS:
;        None.
; SIDE EFFECTS:
;        A file is created, or overwritten.
; RESTRICTIONS:
;        If array is not of type byte, integer, or long, then a conversion
;        to type integer is done. No scaling is made beforehand, so significant
;        loss of information is possible.
;        Only a very simple FITS header can be created.
; MODIFICATION HISTORY:
;        JPW, Nov, 1989.
;-

; open FITS file

if n_elements(fnum) ne 0 then file = filnam+'.'+string(format='(i0)',fnum) $
   else file = filnam
get_lun,unit
openw,unit,file

; check data type, and make conversion if necessary

type = size(data)
case type(type(0)+1) of
   1 : begin
       bitpix = 8
       print,'Note : FITS image is 8 bit deep (BITPIX = 8) '
       end
   2 : bitpix = 16
   3 : begin
       bitpix = 32
       print,'Note : FITS image is 32 bit deep (BITPIX = 32) '
       end
   else : begin
          bitpix = 16
          datb = fix(data)
          print,'Note : data converted to 16 bit integer, no scaling made '
          endelse
endcase

; create header

; SIMPLE
skey = string(format='("SIMPLE  = ",19x,a1)','T')
bkey = bytarr(80) + 32b                            ; 32b = blank
bkey(0) = byte(skey)
head = bkey
; BITPIX
skey = string(format='("BITPIX  = ",i20)',bitpix)
bkey = bytarr(80) + 32b
bkey(0) = byte(skey)
head = [[head],[bkey]]
; NAXIS
skey = string(format='("NAXIS   = ",i20)',type(0))
bkey = bytarr(80) + 32b
bkey(0) = byte(skey)
head = [[head],[bkey]]
; NAXISi
for i=1,type(0) do begin
   skey = string(format='("NAXIS",i1,"  = ",i20)',i,type(i))
   bkey = bytarr(80) + 32b
   bkey(0) = byte(skey)
   head = [[head],[bkey]]
endfor
; DATE-OBS
if n_elements(date) eq 1 then begin
   skey = "DATE-OBS= '"+strmid(date,0,18)+"'"
   bkey = bytarr(80) + 32b
   bkey(0) = byte(skey)
   head = [[head],[bkey]]
endif
; TIME-OBS
if n_elements(time) eq 1 then begin
   skey = "TIME-OBS= '"+strmid(time,0,18)+"'"
   bkey = bytarr(80) + 32b
   bkey(0) = byte(skey)
   head = [[head],[bkey]]
endif
; HISTORY
if n_elements(his) eq 1 then begin
   skey = "HISTORY "+strmid(his,0,72)
   bkey = bytarr(80) + 32b
   bkey(0) = byte(skey)
   head = [[head],[bkey]]
endif
; COMMENT
if n_elements(com) eq 1 then begin
   skey = "COMMENT "+strmid(com,0,72)
   bkey = bytarr(80) + 32b
   bkey(0) = byte(skey)
   head = [[head],[bkey]]
endif
; END
skey = 'END'
bkey = bytarr(80) + 32b
bkey(0) = byte(skey)
head = [[head],[bkey]]
; add blank lines to fill last FITS header record
i = (3600-n_elements(head)/80) mod 36
if i ne 0 then begin
   bkey = bytarr(80,i) + 32b
   head = [[head],[bkey]]
endif

; write the stuff

writeu,unit,head
if type(type(0)+1) gt 3 then writeu,unit,datb else writeu,unit,data
; add null data to fill last FITS data record
i = (n_elements(data)*(bitpix/8)) mod 2880
i = (2880-i) mod 2880
if i gt 0 then begin
   datb = bytarr(i)
   writeu,unit,datb
endif
free_lun,unit
return
end

